@@ -13,6 +13,7 @@
 #               5) http://www.cybercom.net/~dcoffin/dcraw/
 #               6) http://forums.dpreview.com/forums/readflat.asp?forum=1012&thread=31350384
 #                  and http://forum.photome.de/viewtopic.php?f=2&t=353&p=742#p740
+#               7) Kai Lappalainen private communication
 #               JD) Jens Duttke private communication
 #------------------------------------------------------------------------------
 
@@ -23,15 +24,16 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.32';
+$VERSION = '1.35';
 
 sub ProcessFujiDir($$$);
 sub ProcessFaceRec($$$);
 
 # the following RAF version numbers have been tested for writing:
 my %testedRAF = (
-    '0100' => 'E550, E900, S5600, S6000fd, S6500fd, HS10/HS11, S200EXR, X100 (all Ver1.00)',
-    '0102' => 'S100FS Ver1.02',
+    '0100' => 'E550, E900, S5600, S6000fd, S6500fd, HS10/HS11, S200EXR, X100, X-S1 Ver1.00',
+    '0102' => 'S100FS, X10 Ver1.02',
+    '0103' => 'IS Pro Ver1.03',
     '0104' => 'S5Pro Ver1.04',
     '0106' => 'S5Pro Ver1.06',
     '0111' => 'S5Pro Ver1.11',
@@ -269,7 +271,13 @@ my %faceCategories = (
             0x11 => 'Snow', #3
             0x12 => 'Fireworks', #3
             0x13 => 'Underwater', #3
+            0x14 => 'Portrait with Skin Correction', #7
             0x16 => 'Panorama', #PH (X100)
+            0x17 => 'Night (tripod)', #7
+            0x18 => 'Pro Low-light', #7
+            0x19 => 'Pro Focus', #7
+            0x1b => 'Dog Face Detection', #7
+            0x1c => 'Cat Face Detection', #7
             0x100 => 'Aperture-priority AE',
             0x200 => 'Shutter speed priority AE',
             0x300 => 'Manual',
@@ -311,6 +319,10 @@ my %faceCategories = (
         Name => 'SequenceNumber',
         Writable => 'int16u',
     },
+    # (0x1150-0x1152 exist only for Pro Low-light and Pro Focus PictureModes)
+    # 0x1150 - Pro Low-light - val=1; Pro Focus - val=2 (ref 7)
+    # 0x1151 - Pro Low-light - val=4 (number of pictures taken?); Pro Focus - val=2,3 (ref 7)
+    # 0x1152 - Pro Low-light - val=1,3,4 (stacked pictures used?); Pro Focus - val=1,2 (ref 7)
     0x1210 => { #2
         Name => 'ColorMode',
         Writable => 'int16u',
@@ -365,14 +377,16 @@ my %faceCategories = (
         Writable => 'int16u',
         PrintHex => 1,
         PrintConv => {
-            0x000 => 'F0/Standard',
+            0x000 => 'F0/Standard (PROVIA)',
             0x100 => 'F1/Studio Portrait',
             0x110 => 'F1a/Studio Portrait Enhanced Saturation',
-            0x120 => 'F1b/Studio Portrait Smooth Skin Tone',
+            0x120 => 'F1b/Studio Portrait Smooth Skin Tone (ASTIA)',
             0x130 => 'F1c/Studio Portrait Increased Sharpness',
-            0x200 => 'F2/Fujichrome',
+            0x200 => 'F2/Fujichrome (Velvia)',
             0x300 => 'F3/Studio Portrait Ex',
             0x400 => 'F4/Velvia',
+            0x500 => 'Pro Neg. Std', #PH (X-Pro1)
+            0x501 => 'Pro Neg. Hi', #PH (X-Pro1)
         },
     },
     0x1402 => { #2
@@ -554,6 +568,67 @@ my %faceCategories = (
         Format => 'int16u',
         Count => 4,
     },
+    0xc000 => {
+        Name => 'RAFData',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::FujiFilm::RAFData',
+            ByteOrder => 'Little-endian',
+        }
+    },
+);
+
+%Image::ExifTool::FujiFilm::RAFData = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    FIRST_ENTRY => 0,
+    # (FujiFilm image dimensions are REALLY confusing)
+    0 => {
+        Name => 'RawImageWidth',
+        Format => 'int32u',
+        ValueConv => '$$self{FujiLayout} ? ($val / 2) : $val',
+    },
+    4 => {
+        Name => 'RawImageHeight',
+        Format => 'int32u',
+        ValueConv => '$$self{FujiLayout} ? ($val * 2) : $val',
+    },
+);
+
+# TIFF IFD-format information stored in FujiFilm RAF images (ref 5)
+%Image::ExifTool::FujiFilm::IFD = (
+    PROCESS_PROC => \&Image::ExifTool::Exif::ProcessExif,
+    GROUPS => { 0 => 'RAF', 1 => 'FujiIFD', 2 => 'Image' },
+    NOTES => 'Tags found in the FujiIFD information of RAF images from some models.',
+    0xf000 => {
+        Name => 'FujiIFD',
+        Groups => { 1 => 'FujiIFD' },
+        Flags => 'SubIFD',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::FujiFilm::IFD',
+            DirName => 'FujiSubIFD',
+            Start => '$val',
+        },
+    },
+    0xf001 => 'RawImageWidth',
+    0xf002 => 'RawImageHeight',
+    0xf003 => 'BitsPerSample',
+    # 0xf004 - values: 4
+    # 0xf005 - values: 1374, 1668
+    # 0xf006 - some sort of flag indicating packed format?
+    0xf007 => {
+        Name => 'StripOffsets',
+        IsOffset => 1,
+        OffsetPair => 0xf008,  # point to associated byte counts
+    },
+    0xf008 => {
+        Name => 'StripByteCounts',
+        OffsetPair => 0xf007,  # point to associated offsets
+    },
+    # 0xf009 - values: 0, 3
+    # 0xf00a-0xf00c ?
+    # 0xf00d - similar to 0xf00e
+    0xf00e => 'WB_GRBLevels',
+    # 0xf00f ?
 );
 
 # information found in FFMV atom of MOV videos
@@ -695,7 +770,7 @@ sub WriteRAF($$)
     }
     # check to make sure this version of RAF has been tested
     unless ($testedRAF{$ver}) {
-        $exifTool->Error("RAF version $ver not yet tested", 1) and return 1;
+        $exifTool->Warn("RAF version $ver not yet tested", 1);
     }
     # read the embedded JPEG
     unless ($raf->Seek($jpos, 0) and $raf->Read($jpeg, $jlen) == $jlen) {
@@ -739,16 +814,16 @@ sub WriteRAF($$)
             return 1;
         }
         # make sure padding is only zero bytes (can be >100k for HS10)
+        # (have seen non-null padding in X-Pro1)
         if ($buff =~ /[^\0]/) {
-            $exifTool->Error('Non-null bytes found in padding');
-            return 1;
+            return 1 if $exifTool->Error('Non-null bytes found in padding', 1);
         }
     }
     # calculate offset difference due to change in JPEG size
     my $ptrDiff = length($outJpeg) + length($pad) - ($jlen + $oldPadLen);
     # update necessary pointers in header
     foreach $offset (0x5c, 0x64, 0x78, 0x80) {
-        last if $offset >= $jpos;    # some versions have a short header
+        last if $offset >= $jpos;   # some versions have a short header
         my $oldPtr = Get32u(\$hdr, $offset);
         next unless $oldPtr;        # don't update if pointer is zero
         Set32u($oldPtr + $ptrDiff, \$hdr, $offset);
@@ -799,9 +874,9 @@ sub ProcessRAF($$)
     $$exifTool{BASE} -= $jpos;
     $exifTool->FoundTag('PreviewImage', \$jpeg) if $rtnVal;
 
-    # extract information from Fuji RAF directories
-    my $num = '';
-    foreach $offset (0x5c, 0x78) {
+    # extract information from Fuji RAF and TIFF directories
+    my ($rafNum, $ifdNum) = ('','');
+    foreach $offset (0x5c, 0x64, 0x78, 0x80) {
         last if $offset >= $jpos;
         unless ($raf->Seek($offset, 0) and $raf->Read($buff, 4)) {
             $warn = 1;
@@ -809,17 +884,30 @@ sub ProcessRAF($$)
         }
         my $start = unpack('N',$buff);
         next unless $start;
-
-        %dirInfo = (
-            RAF      => $raf,
-            DirStart => $start,
-        );
-        $$exifTool{SET_GROUP1} = "RAF$num";
-        my $tagTablePtr = GetTagTable('Image::ExifTool::FujiFilm::RAF');
-        $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr) or $warn = 1;
-        delete $$exifTool{SET_GROUP1};
-
-        $num = ($num || 1) + 1;
+        if ($offset == 0x64 or $offset == 0x80) {
+            # parse FujiIFD directory
+            %dirInfo = (
+                RAF  => $raf,
+                Base => $start,
+            );
+            $$exifTool{SET_GROUP1} = "FujiIFD$ifdNum";
+            my $tagTablePtr = GetTagTable('Image::ExifTool::FujiFilm::IFD');
+            # this is TIFF-format data only for some models, so no warning if it fails
+            $exifTool->ProcessTIFF(\%dirInfo, $tagTablePtr, \&Image::ExifTool::ProcessTIFF);
+            delete $$exifTool{SET_GROUP1};
+            $ifdNum = ($ifdNum || 1) + 1;
+        } else {
+            # parse RAF directory
+            %dirInfo = (
+                RAF      => $raf,
+                DirStart => $start,
+            );
+            $$exifTool{SET_GROUP1} = "RAF$rafNum";
+            my $tagTablePtr = GetTagTable('Image::ExifTool::FujiFilm::RAF');
+            $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr) or $warn = 1;
+            delete $$exifTool{SET_GROUP1};
+            $rafNum = ($rafNum || 1) + 1;
+        }
     }
     $warn and $exifTool->Warn('Possibly corrupt RAF information');
 
@@ -846,7 +934,7 @@ FujiFilm maker notes in EXIF information, and to read/write FujiFilm RAW
 
 =head1 AUTHOR
 
-Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
