@@ -20,7 +20,7 @@ sub ProcessGE2($$$);
 sub WriteUnknownOrPreview($$$);
 sub FixLeicaBase($$;$);
 
-$VERSION = '1.69';
+$VERSION = '1.73';
 
 my $debug;          # set to 1 to enable debugging code
 
@@ -108,6 +108,16 @@ my $debug;          # set to 1 to enable debugging code
             FixOffsets => '$valuePtr -= 210 if $tagID >= 0x1303',
        },
     },
+    {
+        Name => 'MakerNoteHasselblad',
+        Condition => '$$self{Make} eq "Hasselblad"',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Unknown::Main',
+            ByteOrder => 'Unknown',
+            Start => '$valuePtr',
+            Base => 0, # (avoids warnings since maker notes are not self-contained)
+        },
+    },
     # (the GE X5 has really messed up EXIF-like maker notes starting with
     #  "GENIC\x0c\0" --> currently not decoded)
     {
@@ -608,6 +618,7 @@ my $debug;          # set to 1 to enable debugging code
     {
         Name => 'MakerNotePentax5',
         # (starts with "PENTAX \0")
+        # used by cameras such as the Q, Optio  S1, RS1500 and WG-1
         Condition => '$$valPt=~/^PENTAX \0/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Pentax::Main',
@@ -890,8 +901,10 @@ sub GetMakerNoteOffset($)
     } elsif ($make =~ /^(Panasonic|JVC)\b/) {
         push @offsets, 0;
     } elsif ($make =~ /^SONY/) {
-        # DSLR and "PREMI" models use an offset of 4
-        if ($model =~ /DSLR/ or $$exifTool{OlympusCAMER}) {
+        # earlier DSLR and "PREMI" models use an offset of 4
+        if ($model =~ /^(DSLR-.*|SLT-A(33|35|55V)|NEX-(3|5|C3|VG10E))$/ or
+            $$exifTool{OlympusCAMER})
+        {
             push @offsets, 4;
         } else {
             push @offsets, 0;
@@ -1098,8 +1111,9 @@ sub FixBase($$)
         if ($$dirInfo{FixBase} and $$dirInfo{FixBase} == 2) {
             return 0 if $diff >=0 and $diff <= 24;
         }
-        # (used for testing to extract differences)
+        # ******** (used for testing to extract differences) ********
         # $exifTool->FoundTag('Diff', $diff);
+        # $exifTool->FoundTag('MakeDiff',$makeDiff);
     }
 #
 # handle entry-based offsets
@@ -1314,6 +1328,8 @@ IFD_TRY: for ($offset=$firstTry; $offset<=$lastTry; $offset+=2) {
                 next unless $format or $count or $index == 0;
                 # patch for Canon EOS 40D firmware 1.0.4 bug: allow zero format for last entry
                 next if $format==0 and $index==$num-1 and $$exifTool{Model}=~/EOS 40D/;
+                # patch for Sony cameras like the DSC-P10 that have invalid MakerNote entries
+                next if $num == 12 and $$exifTool{Make} eq 'SONY' and $index >= 8;
                 # (would like to verify tag ID, but some manufactures don't
                 #  sort entries in order of tag ID so we don't have much of
                 #  a handle to verify this field)
@@ -1513,7 +1529,7 @@ maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
