@@ -17,7 +17,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.04';
+$VERSION = '1.05';
 
 sub ProcessComments($$$);
 
@@ -142,6 +142,9 @@ sub ProcessComments($$$)
         if (defined $num) {
             $buff =~ /(.*?)=(.*)/s or last;
             ($tag, $val) = ($1, $2);
+            # Vorbis tag ID's are all capitals, so they may conflict with our internal tags
+            # --> protect against this by adding a trailing underline if necessary
+            $tag .= '_' if $Image::ExifTool::specialTags{$tag};
         } else {
             $tag = 'vendor';
             $val = $buff;
@@ -155,7 +158,7 @@ sub ProcessComments($$$)
             # remove invalid characters in tag name and capitalize following letters
             $name =~ s/[^\w-]+(.?)/\U$1/sg;
             $name =~ s/([a-z0-9])_([a-z])/$1\U$2/g;
-            Image::ExifTool::AddTagToTable($tagTablePtr, $tag, { Name => $name });
+            AddTagToTable($tagTablePtr, $tag, { Name => $name });
         }
         $exifTool->HandleTag($tagTablePtr, $tag, $exifTool->Decode($val, 'UTF8'),
             Index   => $index,
@@ -191,7 +194,7 @@ information from Ogg Vorbis audio headers.
 
 =head1 AUTHOR
 
-Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
