@@ -289,7 +289,7 @@ sub WriteNewTags($$$)
         my $tagInfo = $$newTags{$tag};
         my $nvHash = $exifTool->GetNewValueHash($tagInfo);
         next unless Image::ExifTool::IsCreating($nvHash);
-        my $val = Image::ExifTool::GetNewValues($nvHash);
+        my $val = $exifTool->GetNewValues($nvHash);
         $exifTool->VerboseValue("+ PostScript:$$tagInfo{Name}", $val);
         Write($outfile, EncodeTag($tag, $val)) or $success = 0;
         ++$exifTool->{CHANGED};
@@ -585,9 +585,9 @@ sub WritePS($$)
                 $val = DecodeComment($val, $raf, \@lines, \$data);
                 $val = join $exifTool->Options('ListSep'), @$val if ref $val eq 'ARRAY';
                 my $nvHash = $exifTool->GetNewValueHash($tagInfo);
-                if (Image::ExifTool::IsOverwriting($nvHash, $val)) {
+                if ($exifTool->IsOverwriting($nvHash, $val)) {
                     $exifTool->VerboseValue("- PostScript:$$tagInfo{Name}", $val);
-                    $val = Image::ExifTool::GetNewValues($nvHash);
+                    $val = $exifTool->GetNewValues($nvHash);
                     ++$exifTool->{CHANGED};
                     next unless defined $val;   # next if tag is being deleted
                     $exifTool->VerboseValue("+ PostScript:$$tagInfo{Name}", $val);
@@ -763,7 +763,7 @@ Thanks to Tim Kordick for his help testing the EPS writer.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
