@@ -274,7 +274,11 @@ sub SetPropertyPath($$;$$$$)
     # set PropertyPath for all flattened tags of this structure if necessary
     # (note: don't do this for variable-namespace structures (undef NAMESPACE))
     my $strTable = $$tagInfo{Struct};
-    if ($strTable and $$strTable{NAMESPACE}) {
+    if ($strTable and $$strTable{NAMESPACE} and not ($parentID and
+        # must test NoSubStruct flag to avoid infinite recursion
+        (($$tagTablePtr{$parentID} and $$tagTablePtr{$parentID}{NoSubStruct}) or
+        length $parentID > 500))) # avoid deep recursion
+    {
         # make sure the structure namespace has been registered
         # (user-defined namespaces may not have been)
         RegisterNamespace($strTable) if ref $$strTable{NAMESPACE};
@@ -295,7 +299,7 @@ sub SetPropertyPath($$;$$$$)
         # (could happen if we were just writing a structure)
         unless ($tagInfo) {
             $tagInfo = { Name => ucfirst($flatID), Flat => 1 };
-            Image::ExifTool::AddTagToTable($tagTablePtr, $flatID, $tagInfo);
+            AddTagToTable($tagTablePtr, $flatID, $tagInfo);
         }
         # set StructType flag if any containing structure has a TYPE
         $$tagInfo{StructType} = 1 if $isType;
@@ -797,7 +801,7 @@ sub WriteXMP($$;$)
                     my $tg = $exifTool->GetGroup($tagInfo, 1) . ':' . $$tagInfo{Name};
                     my $wrn = lc($path) eq lc($matches[0]) ? 'tag ID case' : 'list type';
                     if ($err) {
-                        $exifTool->Warn("Incorrect $wrn for $tg conflicts with existing tag");
+                        $exifTool->Warn("Incorrect $wrn for existing $tg (not changed)");
                     } else {
                         # fix the incorrect property paths for all values of this tag
                         foreach $match (@matches) {
@@ -1010,6 +1014,17 @@ sub WriteXMP($$;$)
             }
             substr($path, $pos, $len) = $idx . $subIdx;
         }
+        # make sure any empty structures are deleted
+        # (ExifTool shouldn't write these, but other software may)
+        if (defined $$tagInfo{Flat}) {
+            my $p = $path;
+            while ($p =~ s/\/[^\/]+$//) {
+                next unless $capture{$p};
+                # it is an error if this property has a value
+                $exifTool->Error("Improperly structured XMP ($p)",1) if $capture{$p}[0] =~ /\S/;
+                delete $capture{$p};    # delete the (hopefully) empty structure
+            }
+        }
     }
     # remove the ExifTool members we created
     delete $exifTool->{XMP_CAPTURE};
@@ -1268,7 +1283,7 @@ This file contains routines to write XMP metadata.
 
 =head1 AUTHOR
 
-Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
